/****************************************************************************
                           PREPLAXBARY.C

This program converts level 2 event fits file, usergti fits file and 
orbit fits file to a single consistent fits file, which is compatible with 
AS1BARY for the analysis of LAXPC data

Uses lcfitsio library

Avishek/BCJ
07/12/2017
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "fitsio.h"

void reportAndExitOnFITSerror(int status) {
	if(status) {
		fits_report_error(stdout, status);
		exit(1);
	}
}


int main(int argc, char * argv[]){
	fitsfile *event, *orbit, *gti, *out;   
	int status = 0;  
	int iomode, hdupos,hdutype,nulltype,hdunum;
	long startobsi,stopobsi;
	double startobs, stopobs, startobsf, stopobsf,startobsid,stopobsid;
	double mjdreff, equinox, rapnt, decpnt;
	long mjdrefi;
	char card[FLEN_CARD],comment[FLEN_COMMENT];
	char phdustr[25][FLEN_CARD];
	char *ttype[]= 			{ "TSTART","TSTARTI","TSTARTF","TSTOP","TSTOPI","TSTOPF","MISSION","INSTRUME","TELESCOP","ORIGIN","TIMESYS","TIMEUNIT",
	"MJDREFI","MJDREFF","OBJECT","RADECSYS","EQUINOX","RA_PNT","DEC_PNT","OBS_ID","DATE-OBS","TIME-OBS","DATE-OBS","DATE-END",
	"TIME-END","TIMEDEL" };
	char *tform[] = {"1D","1V","1D","1D","1V","1D","16A","16A","16A","16A","16A","16A","1V","1D","32A","32A","1D","1D","1D",
"32A","32A","32A","32A","32A","32A","1D"};
	char *tunit[] = {"s","s","s","s","s","s"," "," "," "," "," ","s ","s "," "," "," "," "," "," "," "," "," "," "," "," "," "," "," "," "};
	char extname[]={"event file"};
	int bitpix   =  8;
	int naxis   =  0;
	long *naxes; // we know for sure that it will be a 2D table	
	double dval;
	int ival;
	long lval;
	// so far so good
	// Read command line arguments
	if (argc != 5)
	{
		printf("Usage:  <program> eventfile orbitfile gtifile outputfile\n");
		printf("\n");
		return(0);
	}
	// Get tstart and tstop from orbit file
  	if (!fits_open_file(&orbit, argv[2], iomode, &status))
    {
     	fits_get_hdu_num(orbit, &hdupos);  // Get the current HDU position
      	if (hdupos == 1) 
		{
          	fits_read_key(orbit,TDOUBLE,"TSTART", &startobs, comment,&status);
          	fits_read_key(orbit,TDOUBLE,"TSTOP", &stopobs, comment,&status);
			reportAndExitOnFITSerror(status);
		}
      	else
			exit(0);
    }
  	fits_close_file(orbit, &status);
	// printf("In orbit file\tstart_time : %lf , stop_time : %lf\n",startobs, stopobs);
	// moving on to events file 
	fits_open_file(&event, argv[1], READONLY, &status);
	// before that editing startobs and stopobs
	startobs = startobs + 3.0;
	stopobs = stopobs - 3.0;
	startobsid = floor(startobs);
	startobsf = startobs - startobsid;
	startobsi = (int) startobsid;
  	stopobsid = floor(stopobs);
  	stopobsf = stopobs - stopobsid;
  	stopobsi = (int) stopobsid;
	// event file io 
	// creating out file rn
	fits_create_file(&out, argv[4], &status); // Creating
	fits_create_img( out, bitpix, naxis, naxes, &status );
	reportAndExitOnFITSerror(status);
	//printf("Are you please coming here?\n");
	// not changing event file in any file 
	//fits_read_key(event,TDOUBLE, "TSTART",&dval,comment,&status);

	fits_update_key( out, TDOUBLE, "TSTART", &startobs, "Start time", &status);

	//fits_read_key(event,TINT, "TSTARTI",&ival,comment,&status);
	fits_update_key( out, TINT, "TSTARTI", &startobsi, "integer portion of start time", &status);

	//fits_read_card(event,"TSTARTF",card,&status);
	fits_update_key( out, TDOUBLE, "TSTARTF", &startobsf, "fractional observation start time", &status);

	//fits_read_card(event,"TSTOP",card,&status);
	fits_update_key( out, TDOUBLE, "TSTOP", &stopobs, "Stop time", &status);

	//fits_read_card(event,"TSTOPI",card,&status);
	fits_update_key( out, TINT, "TSTOPI", &stopobsi, "integer observation stop time", &status);

	//fits_read_card(event,"TSTOPF",card,&status);
	fits_update_key( out, TDOUBLE, "TSTOPF", &stopobsf, "fractional observation stop time", &status);
	// from here on, we will be using fits_read_key
	char mission[FLEN_CARD],instru[FLEN_CARD],teles[FLEN_CARD],origin[FLEN_CARD],timesys[FLEN_CARD],timeunit[FLEN_CARD];
	long timezeri, orb_num;
	double timezerf;
	char object[FLEN_CARD],sourceid[FLEN_CARD],targetid[FLEN_CARD],radecsys[FLEN_CARD],obsid[FLEN_CARD];
	char dateobs[FLEN_CARD],timeobs[FLEN_CARD],dateend[FLEN_CARD],timeend[FLEN_CARD];
	fits_read_key(event, TSTRING, "MISSION",mission,comment,&status);
	fits_read_key(event, TSTRING, "INSTRUME", instru,comment, &status);
	fits_read_key(event, TSTRING, "TELESCOP", teles,comment, &status);
	fits_read_key(event, TSTRING, "ORIGIN", origin,comment, &status);
	fits_read_key(event, TSTRING, "TIMESYS", timesys,comment, &status);
	fits_read_key(event, TSTRING, "TIMEUNIT", timeunit,comment, &status);
	fits_read_key(event, TLONG, "MJDREFI", &mjdrefi,comment, &status);
	fits_read_key(event, TLONG, "TIMEZERI", &timezeri,comment, &status);
	fits_read_key(event, TLONG, "ORB_NUM", &orb_num,comment, &status);
	fits_read_key(event, TDOUBLE, "MJDREFF", &mjdreff,comment, &status);
	fits_read_key(event, TDOUBLE, "TIMEZERF", &timezerf,comment, &status);
	fits_read_key(event, TSTRING, "OBJECT", object,comment, &status);
	fits_read_key(event, TSTRING, "SOURCEID", sourceid,comment, &status);
	fits_read_key(event, TSTRING, "TARGETID", targetid,comment, &status);
	fits_read_key(event, TSTRING, "RADECSYS", radecsys,comment, &status);
	fits_read_key(event, TDOUBLE, "EQUINOX", &equinox,comment, &status);
	fits_read_key(event, TDOUBLE, "RA_PNT", &rapnt,comment, &status);
	fits_read_key(event, TDOUBLE, "DEC_PNT", &decpnt,comment, &status);
	fits_read_key(event, TSTRING, "OBS_ID", obsid,comment, &status);
	fits_read_key(event, TSTRING, "DATE-OBS", dateobs,comment, &status);
	fits_read_key(event, TSTRING, "TIME-OBS", timeobs,comment, &status);
	fits_read_key(event, TSTRING, "DATE-END", dateend,comment, &status);
	fits_read_key(event, TSTRING, "TIME-END", timeend,comment, &status);
	
	// god knows why it is expecting this to be here
	fits_update_key(out, TSTRING, "INSTRUME", instru,comment, &status);
	fits_update_key(out, TSTRING, "TELESCOP", teles,comment, &status);
	fits_update_key(out, TSTRING, "ORIGIN", origin,comment, &status);
	fits_update_key(out, TSTRING, "TIMESYS", timesys,comment, &status);
	fits_update_key(out, TSTRING, "TIMEUNIT", timeunit,comment, &status);
	fits_update_key(out, TLONG, "MJDREFI", &mjdrefi,comment, &status);
	fits_update_key(out, TLONG, "TIMEZERI", &timezeri,comment, &status);
	fits_update_key(out, TLONG, "ORB_NUM", &orb_num,comment, &status);
	fits_update_key(out, TDOUBLE, "MJDREFF", &mjdreff,comment, &status);
	fits_update_key(out, TDOUBLE, "TIMEZERF", &timezerf,comment, &status);
	fits_update_key(out, TSTRING, "OBJECT", object,comment, &status);
	fits_update_key(out, TSTRING, "SOURCEID", sourceid,comment, &status);
	fits_update_key(out, TSTRING, "TARGETID", targetid,comment, &status);
	fits_update_key(out, TSTRING, "RADECSYS", radecsys,comment, &status);
	fits_update_key(out, TDOUBLE, "EQUINOX", &equinox,comment, &status);
	fits_update_key(out, TDOUBLE, "RA_PNT", &rapnt,comment, &status);
	fits_update_key(out, TDOUBLE, "DEC_PNT", &decpnt,comment, &status);
	fits_update_key(out, TSTRING, "OBS_ID", obsid,comment, &status);
	fits_update_key(out, TSTRING, "DATE-OBS", dateobs,comment, &status);
	fits_update_key(out, TSTRING, "TIME-OBS", timeobs,comment, &status);
	fits_update_key(out, TSTRING, "DATE-END", dateend,comment, &status);
	fits_update_key(out, TSTRING, "TIME-END", timeend,comment, &status);

	dval = 0.00001;
	fits_update_key( out, TDOUBLE, "TIMEDEL", &dval, "Time resolution", &status);

	fits_movabs_hdu(event,2,&iomode,&status);
	fits_copy_file(event,out,0,1,0,&status);
	fits_movabs_hdu( event, 1, NULL, &status );
	// here u put those 
	fits_update_key(out, TSTRING, "MISSION",mission,comment,&status);

	//fits_select_rows(event, out, "(TSTART .GE. startobs) .AND. (TSTOP .LE. stopobs)", &status );
	reportAndExitOnFITSerror(status);

	fits_update_key( out, TDOUBLE, "TSTART", &startobs, "Start time", &status);
	fits_update_key( out, TINT, "TSTARTI", &startobsi, "integer portion of start time", &status);
	fits_update_key( out, TDOUBLE, "TSTARTF", &startobsf, "fractional observation start time", &status);
	fits_update_key( out, TDOUBLE, "TSTOP", &stopobs, "Stop time", &status);
	fits_update_key( out, TINT, "TSTOPI", &stopobsi, "integer observation stop time", &status);
	fits_update_key( out, TDOUBLE, "TSTOPF", &stopobsf, "fractional observation stop time", &status);
	fits_update_key(out, TSTRING, "INSTRUME", instru,comment, &status);
	fits_update_key(out, TSTRING, "TELESCOP", teles,comment, &status);
	fits_update_key(out, TSTRING, "ORIGIN", origin,comment, &status);
	fits_update_key(out, TSTRING, "TIMESYS", timesys,comment, &status);
	fits_update_key(out, TSTRING, "TIMEUNIT", timeunit,comment, &status);
	fits_update_key(out, TLONG, "MJDREFI", &mjdrefi,comment, &status);
	fits_update_key(out, TLONG, "TIMEZERI", &timezeri,comment, &status);
	fits_update_key(out, TLONG, "ORB_NUM", &orb_num,comment, &status);
	fits_update_key(out, TDOUBLE, "MJDREFF", &mjdreff,comment, &status);
	fits_update_key(out, TDOUBLE, "TIMEZERF", &timezerf,comment, &status);
	fits_update_key(out, TSTRING, "OBJECT", object,comment, &status);
	fits_update_key(out, TSTRING, "SOURCEID", sourceid,comment, &status);
	fits_update_key(out, TSTRING, "TARGETID", targetid,comment, &status);
	fits_update_key(out, TSTRING, "RADECSYS", radecsys,comment, &status);
	fits_update_key(out, TDOUBLE, "EQUINOX", &equinox,comment, &status);
	fits_update_key(out, TDOUBLE, "RA_PNT", &rapnt,comment, &status);
	fits_update_key(out, TDOUBLE, "DEC_PNT", &decpnt,comment, &status);
	fits_update_key(out, TSTRING, "OBS_ID", obsid,comment, &status);
	fits_update_key(out, TSTRING, "DATE-OBS", dateobs,comment, &status);
	fits_update_key(out, TSTRING, "TIME-OBS", timeobs,comment, &status);
	fits_update_key(out, TSTRING, "DATE-END", dateend,comment, &status);
	fits_update_key(out, TSTRING, "TIME-END", timeend,comment, &status);
	//fits_delete_key( out, "NAXIS", &status);
	//fits_delete_key( out, "NAXIS2", &status);
	reportAndExitOnFITSerror(status);
	char comm[1000];
	sprintf(comm,"(TIME .GE. %lf) .AND. (TIME .LE. %lf)",startobs,stopobs);
	//printf("First selection\n");
	fits_select_rows(out, out, comm, &status );

// Add  extension header for GTI 

	fits_open_file(&gti, argv[3], READONLY, &status);
	reportAndExitOnFITSerror(status);
	fits_get_num_hdus(gti,&hdunum,&status);
	
	fits_movabs_hdu(gti,2,&iomode,&status);
	fits_copy_file(gti,out,0,1,0,&status);
	//printf("Second selection\n");
	sprintf(comm,"(START .GE. %lf) .AND. (STOP .LE. %lf)",startobs,stopobs);
	fits_select_rows(out, out, comm, &status );
	reportAndExitOnFITSerror(status);
	fits_update_key( out, TDOUBLE, "TSTART", &startobs, "Start time", &status);
	fits_update_key( out, TINT, "TSTARTI", &startobsi, "integer portion of start time", &status);
	fits_update_key( out, TDOUBLE, "TSTARTF", &startobsf, "fractional observation start time", &status);
	fits_update_key( out, TDOUBLE, "TSTOP", &stopobs, "Stop time", &status);
	fits_update_key( out, TINT, "TSTOPI", &stopobsi, "integer observation stop time", &status);
	fits_update_key( out, TDOUBLE, "TSTOPF", &stopobsf, "fractional observation stop time", &status);
	fits_update_key(out, TSTRING, "INSTRUME", instru,comment, &status);
	fits_update_key(out, TSTRING, "TELESCOP", teles,comment, &status);
	fits_update_key(out, TSTRING, "ORIGIN", origin,comment, &status);
	fits_update_key(out, TSTRING, "TIMESYS", timesys,comment, &status);
	fits_update_key(out, TSTRING, "TIMEUNIT", timeunit,comment, &status);
	fits_update_key(out, TLONG, "MJDREFI", &mjdrefi,comment, &status);
	fits_update_key(out, TLONG, "TIMEZERI", &timezeri,comment, &status);
	fits_update_key(out, TLONG, "ORB_NUM", &orb_num,comment, &status);
	fits_update_key(out, TDOUBLE, "MJDREFF", &mjdreff,comment, &status);
	fits_update_key(out, TDOUBLE, "TIMEZERF", &timezerf,comment, &status);
	fits_update_key(out, TSTRING, "OBJECT", object,comment, &status);
	fits_update_key(out, TSTRING, "SOURCEID", sourceid,comment, &status);
	fits_update_key(out, TSTRING, "TARGETID", targetid,comment, &status);
	fits_update_key(out, TSTRING, "RADECSYS", radecsys,comment, &status);
	fits_update_key(out, TDOUBLE, "EQUINOX", &equinox,comment, &status);
	fits_update_key(out, TDOUBLE, "RA_PNT", &rapnt,comment, &status);
	fits_update_key(out, TDOUBLE, "DEC_PNT", &decpnt,comment, &status);
	fits_update_key(out, TSTRING, "OBS_ID", obsid,comment, &status);
	fits_update_key(out, TSTRING, "DATE-OBS", dateobs,comment, &status);
	fits_update_key(out, TSTRING, "TIME-OBS", timeobs,comment, &status);
	fits_update_key(out, TSTRING, "DATE-END", dateend,comment, &status);
	fits_update_key(out, TSTRING, "TIME-END", timeend,comment, &status);
	fits_update_key(out, TSTRING, "EXTNAME", "GTI",comment, &status); // change extension name
	fits_close_file(gti, &status);
	//printf("or here?\n");
	reportAndExitOnFITSerror(status);
	fits_close_file(event, &status);
	//printf("or or here?\n");
	reportAndExitOnFITSerror(status);
	fits_close_file(out, &status);
	reportAndExitOnFITSerror(status);
	return 0;
}
